Package: OmicNavigator
Type: Package
Title: Open-Source Software for 'Omic' Data Analysis and Visualization
Description: 
  A tool for interactive exploration of the results from 'omics'
  experiments to facilitate novel discoveries from high-throughput biology. The
  software includes R functions for the 'bioinformatician' to deposit study
  metadata and the outputs from statistical analyses (e.g. differential
  expression, enrichment). These results are then exported to an interactive
  JavaScript dashboard that can be interrogated on the user's local machine or
  deployed online to be explored by collaborators. The dashboard includes
  'sortable' tables, interactive plots including network visualization, and
  fine-grained filtering based on statistical significance.
Version: 1.19.0
Authors@R: c(
  person("Terrence", "Ernst", role = c("aut"),
         comment = "Web application"),
  person("John", "Blischak", role = c("aut"),
         email = "jdblischak@gmail.com",
         comment = c(ORCID = "0000-0003-2634-9879")),
  person("Paul", "Nordlund", role = c("aut"),
         comment = "Web application"),
  person("Justin", "Moore", role = c("aut"),
         comment = "UpSet-related functions and web application"),
  person("Joe", "Dalen", role = c("aut"),
         comment = "Barcode functionality and web application"),
  person("Akshay", "Bhamidipati", role = c("aut"),
         comment = "Web application"),
  person("Brett", "Engelmann", role = c("aut", "cre"),
         email = "brett.engelmann@abbvie.com"),
  person("Marco", "Curado", role = c("aut"),
         comment = "Improved plotting capabilities"),
  person("Joe", "LoGrasso", role = c("aut"),
         comment = "Support for plotly"),
  person("Elyse", "Geoffroy", role = c("ctb")),
  person(given = "AbbVie Inc.", role = c("cph", "fnd"))
  )
URL: https://github.com/abbvie-external/OmicNavigator
BugReports: https://github.com/abbvie-external/OmicNavigator/issues
License: MIT + file LICENSE
License_restricts_use: no
License_is_FOSS: yes
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.2.0)
Imports: data.table (>= 1.12.4), graphics, jsonlite, stats, tools,
        utils
Suggests: faviconPlease, ggplot2, opencpu, plotly, tinytest (>= 1.2.3),
        ttdo (>= 0.0.6), UpSetR
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-16 21:08:51 UTC; engelbw
Author: Terrence Ernst [aut] (Web application),
  John Blischak [aut] (ORCID: <https://orcid.org/0000-0003-2634-9879>),
  Paul Nordlund [aut] (Web application),
  Justin Moore [aut] (UpSet-related functions and web application),
  Joe Dalen [aut] (Barcode functionality and web application),
  Akshay Bhamidipati [aut] (Web application),
  Brett Engelmann [aut, cre],
  Marco Curado [aut] (Improved plotting capabilities),
  Joe LoGrasso [aut] (Support for plotly),
  Elyse Geoffroy [ctb],
  AbbVie Inc. [cph, fnd]
Maintainer: Brett Engelmann <brett.engelmann@abbvie.com>
Repository: CRAN
Date/Publication: 2025-12-17 07:50:08 UTC
Built: R 4.6.0; ; 2026-04-25 20:56:16 UTC; unix
