# Generated by roxygen2: do not edit by hand

S3method(c,onStudy)
S3method(getElements,character)
S3method(getElements,default)
S3method(getElements,onStudy)
S3method(print,onStudy)
S3method(summary,onStudy)
export(addAnnotations)
export(addAssays)
export(addBarcodes)
export(addEnrichments)
export(addEnrichmentsLinkouts)
export(addFeatures)
export(addMapping)
export(addMetaAssays)
export(addMetaFeatures)
export(addMetaFeaturesLinkouts)
export(addModels)
export(addObjects)
export(addOverlaps)
export(addPlots)
export(addReports)
export(addResults)
export(addResultsLinkouts)
export(addSamples)
export(addTests)
export(combineStudies)
export(createStudy)
export(exportStudy)
export(getAnnotations)
export(getAssays)
export(getBarcodeData)
export(getBarcodes)
export(getEnrichments)
export(getEnrichmentsAnnotations)
export(getEnrichmentsIntersection)
export(getEnrichmentsLinkouts)
export(getEnrichmentsModels)
export(getEnrichmentsNetwork)
export(getEnrichmentsStudies)
export(getEnrichmentsTable)
export(getEnrichmentsUpset)
export(getFavicons)
export(getFeatures)
export(getInstalledStudies)
export(getLinkFeatures)
export(getMapping)
export(getMetaAssays)
export(getMetaFeatures)
export(getMetaFeaturesLinkouts)
export(getMetaFeaturesTable)
export(getModels)
export(getNodeFeatures)
export(getObjects)
export(getOverlaps)
export(getPackageVersion)
export(getPlots)
export(getPlottingData)
export(getReportLink)
export(getReports)
export(getResults)
export(getResultsIntersection)
export(getResultsLinkouts)
export(getResultsModels)
export(getResultsStudies)
export(getResultsTable)
export(getResultsTests)
export(getResultsUpset)
export(getSamples)
export(getStudyMeta)
export(getTests)
export(getUpsetCols)
export(importStudy)
export(installApp)
export(installStudy)
export(plotStudy)
export(removeStudy)
export(startApp)
export(validateStudy)
importFrom(data.table,"%chin%")
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,as.data.table)
importFrom(data.table,chmatch)
importFrom(data.table,data.table)
importFrom(data.table,dcast.data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,melt)
importFrom(data.table,merge.data.table)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(jsonlite,read_json)
importFrom(jsonlite,write_json)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(tools,file_ext)
importFrom(utils,capture.output)
importFrom(utils,download.file)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,remove.packages)
importFrom(utils,unzip)
